/*
** JDBCExample (c) 2003 DataDirect Technologies, Inc.
**
** Coded by Robb Oliver
**
** DISCLAIMER: The sample code is not supported under any
** DataDirect Technologies support program or service. The
** sample code is provided on an AS IS basis.  DataDirect
** Technologies makes no warranties, express or implied, and
** disclaims all implied warranties including, without
** limitation, the implied warranties of merchantability or of
** fitness for a particular purpose. The entire risk arising 
** out of the use or performance of the sample code is borne
** by the user. In no event shall DataDirect Technologies, its
** employees, or anyone else involved in the creation, 
** production, or delivery of the code be liable for any 
** damages whatsoever (including, without limitation, damages
** for loss of business profits, business interruption, loss
** of business information, or other pecuniary loss) arising
** out of the use of or inability to use the sample code, even
** if DataDirect Technologies has been advised of the 
** possibility of such damages.
**                      
*/

import java.io.*;
import java.sql.*;

public class JDBCExample {

	public static void main (String[] args) {

		String strDB = "";
		String strDrvURL = "";

		//*NOTE* The Buffered Reader may not work with all extended characters
		BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));

		try {

			if (args.length < 1) {

				System.out.print("Driver URL : ");
				strDrvURL = stdin.readLine();
			}
			else {

				strDrvURL = args[0];
			}

			if (args.length < 2) {

				System.out.print("Connection URL : ");
				strDB = stdin.readLine();
			}
			else {

				strDB = args[1];
			}
		}
		catch(IOException e) {

			handleException(e);
		}
		
		try {

			System.out.println();
			System.out.print("Registering driver...");
			Class.forName(strDrvURL).newInstance();
			System.out.println("registered.");
		}
		catch (Exception e) {

			handleException(e);
			return;
		}

		Connection con = null;

		try {

			System.out.print("Making connection...");
			con = DriverManager.getConnection(strDB);
			System.out.println("connected.");
		}
		catch (Exception e) {

			handleException(e);
			return;
		}

		printWarnings(con);

		System.out.print("Getting Database Metadata...");
		DatabaseMetaData dbmd;

		try {

			dbmd = con.getMetaData();
			
		}
		catch (Exception e) {

			handleException(e);
			return;
		}
		System.out.println("done.");

		System.out.print("Displaying Specific Meta Data...\n");

		try {

			System.out.println();
			System.out.println("JVM    : " + System.getProperty ("java.vm.version"));
			System.out.println("Driver : " + dbmd.getDriverName() + " " + dbmd.getDriverMajorVersion() + "." + dbmd.getDriverMinorVersion() + " (" + dbmd.getDriverVersion() + ")");
			System.out.println("URL    : " + dbmd.getURL());
			System.out.println("DBMS   : " + dbmd.getDatabaseProductName() + " " + dbmd.getDatabaseMajorVersion() + "." + dbmd.getDatabaseMinorVersion() + " (" + dbmd.getDatabaseProductVersion() + ")");
			System.out.println();
		}
		catch (Exception e) {

			handleException(e);
			return;
		}
		System.out.println("done.\n");
		
		System.out.print("Creating Statement...");
		Statement stmt;

		try {

			stmt = con.createStatement();
			System.out.println("created.\n");
		}
		catch (Exception e) {

			handleException(e);
			return;
		}

		String strSQL = "";

		while (true) {

			//Read in SQL statement
			System.out.print("SQL> ");
			try {

				strSQL = stdin.readLine();
				
				if (strSQL.equals("")) {
					
					break;
				}
			}
			catch (Exception e) {

				handleException(e);
				return;
			}

			//Execute stmt
			try {

				stmt.execute(strSQL);
			}
			catch (Exception e) {

				handleException(e);
				continue;
			}

                        boolean moreResults = true;

			//get the results back
			while (moreResults) {

				try {
					
		                        int updateCount;
		                        updateCount = stmt.getUpdateCount();
		                        if (updateCount > 0) {
		
						System.out.println("Rows updated : " + updateCount);
						
						stmt.getMoreResults();
						continue;
		                	}
		                	if (updateCount == 0) {
		
						System.out.println("Rows updated : " + 0);
						
						stmt.getMoreResults();
						continue;
		              		}
		
					ResultSet results = stmt.getResultSet();
		
					if (results != null) {
		
			                        System.out.println("Result set returned:");
			                        
						ResultSetMetaData rsmd = results.getMetaData();
		
						int numCols = rsmd.getColumnCount();
						int i, rowcount = 0;
						StringBuffer buf = new StringBuffer();
		
						// get column header info
						for (i=1; i <= numCols; i++) {
		
							if (i > 1) {
		
								buf.append(",");
							}
		
							buf.append(rsmd.getColumnLabel(i));
						}
						buf.append("\n");
						
						System.out.print(buf);
						buf = new StringBuffer("");
		
						while (results.next()) {
		
							// Loop through each column, getting the column
							// data and displaying
							for (i=1; i <= numCols; i++) {
		
								if (i > 1) {
		
									buf.append(", ");
								}

								buf.append(results.getString(i));
							}

							buf.append("\n");
		
							rowcount++;
						}
		
						System.out.println(buf);
		
						System.out.println("Rows : " + rowcount);
		
						stmt.getMoreResults();
						continue;
		
					} //if (results != null)
					else {
		
						System.out.println("No more Results");
						moreResults = false; //no more results
					}
				}
				catch (Exception e) {
				
					handleException(e);
					break;
				}
			} //while (moreResults)

			// At this point it would be safe to call getXXX methods for registered OUT parameters
			// but this will not be addressed in this example

		} //while (sql loop)

		System.out.print("\nClosing connection...");
		try {

			con.close();

			System.out.println("closed.\n");
		}
		catch (Exception e) {

			handleException(e);
		}
	} //main

	public static void printWarnings(Connection con) {

		SQLWarning warning = null;

		try {

			warning = con.getWarnings();

			if (warning != null) {
		
				System.out.println();

				while (warning != null) {

					System.out.println("Warning: "+warning);
					warning = warning.getNextWarning();
				}
				
				System.out.println();
			}
		}
		catch (Exception e) {

			handleException(e);
		}
	} //printWarnings

	public static void handleException(Exception e) {

		System.out.println("\n" + e);

		if (e instanceof SQLException) {

			System.out.println("SQLState =" + ((SQLException)e).getSQLState());
			System.out.println("ErrorCode=" + ((SQLException)e).getErrorCode() + "\n");
		}

		e.printStackTrace();
		System.out.println();
	} //handleException
} //JDBCExample

